
void veccopy(double r1[2], double r2[2])
{
	int i;
	for(i=0; i<2; i++) r2[i]=r1[i];
}

void vecadd(double r1[2], double r2[2], double r3[2])
{
	int i;
	for(i=0; i<2; i++) r3[i]=r1[i]+r2[i];
}

void vecsub(double r1[2], double r2[2], double r3[2])
{
	int i;
	for(i=0; i<2; i++) r3[i]=r1[i]-r2[i];
}

void vecprod(double r1[2], double x, double r2[2])
{
	r2[0]=x*r1[0];
	r2[1]=x*r1[1];
}

void vecdiv(double r1[2], double x, double r2[2])
{
	r2[0]=r1[0]/x;
	r2[1]=r1[1]/x;
}

void veczero(double r[2])
{
	r[0]=r[1]=0.0;
}

void vecswap(double r1[2], double r2[2])
{
	double r[2];
	veccopy(r1, r);
	veccopy(r2, r1);
	veccopy(r, r2);
}

double dotprod(double r1[2], double r2[2])
{
	return r1[0]*r2[0]+r1[1]*r2[1];
}

double unitdotprod(double r1[2], double r2[2])
{
	double n1, n2;
	if((n1=norm(r1))==0.0) return 0.0;
	if((n2=norm(r2))==0.0) return 0.0;
	return dotprod(r1,r2)/n1/n2;
}

double crossprod(double r1[2], double r2[2])
{
	return r1[0]*r2[1]-r2[0]*r1[1];
}

double norm(double r[2])
{
	return sqrt(r[0]*r[0]+r[1]*r[1]);
}

void normalize(double r1[2], double r2[2])
{
	int i;
	double r;
	r=norm(r1);
	if(r==0.0) r2[0]=r2[1]=0.0;
	else {
		for(i=0; i<2; i++) r2[i]=r1[i]/r;
	}
}

double distance(double r1[2], double r2[2])
{
	return sqrt(pow(r1[0]-r2[0],2)+pow(r1[1]-r2[1],2));
}

double distancesquare(double r1[2], double r2[2])
{
	return pow(r1[0]-r2[0],2)+pow(r1[1]-r2[1],2);
}

void limitvector(double r[2], double nrm)	// s.t. 0<=|r|<=nrm
{
	double nr, fct;
	nr=norm(r);
	if(nr<=nrm) return;
	fct=nrm/nr;	// 0<=fct<1
	vecprod(r, fct, r);
}

double randsign(void)
{
	return ran2(&mseed)>0.5?1.0:-1.0;
}

double getangle(double r1[2], double r2[2])	// 0<=angle<pi
{
	double dotp, denom;
	dotp=dotprod(r1,r2);
	denom=norm(r1)*norm(r2);
	if(denom==0.0) return 0.0;
	else if(dotp>denom) return 0.0;
	else if(dotp<-denom) return Pi;
	else return acos(dotp/denom);
}

double getanglesign(double r1[2], double r2[2])	// angle with sign
{
	double crossp;
	crossp=crossprod(r1, r2);
	return getangle(r1, r2)*sign(crossp);	// CCW being +, CW being -
}

void limitpi(double *x)	// limit -pi<x<pi
{
	while(*x>Pi) *x-=Pidb;
	while(*x<-Pi) *x+=Pidb;
}

double avoidzero(double x)
{
	if(fabs(x)<1.0e-9) return sign(x)*1.0e-9;
	else return x;
}

void Crt2Pola(double r[2], double *len, double *a)	// CRT to polar, 0<=a<2*Pi
{
	double xy;
	xy=sqrt(r[0]*r[0]+r[1]*r[1]);
	*len=xy;
	if(xy==0.0) *a=0.0;
	else if(r[1]>=0.0) *a=acos(r[0]/xy);	// 1st & 2nd quadrants
	else *a=Pidb-acos(r[0]/xy);	// 3rd & 4th quadrants
}

void Crt2Polb(double r[2], double *len, double *a)	// CRT to polar, -pi<a<Pi
{
	double xy;
	xy=sqrt(r[0]*r[0]+r[1]*r[1]);
	*len=xy;
	if(xy==0.0) *a=0.0;
	else if(r[1]>=0) *a=asin(r[0]/xy);	// 1st & 2nd quadrants
	else if(r[0]>=0) *a=Pi-asin(r[0]/xy);	// 4th quadrant
	else *a=-Pi-asin(r[0]/xy);	// 3rd quadrant
}

void Pol2Crt(double len, double a, double r[2])	// polar to CRT, r is a unit vector
{
	r[0]=len*cos(a);
	r[1]=len*sin(a);
}


void getnormal(double r1[2], double r2[2], double r3[2], double nrm[2])	// more stable than the other?
{	// find the outword normal of curve r1-r2-r3 at r2. r1-r2-r3 is CW!
	double dr12[2], dr23[2], nrm12[2], nrm23[2];
	
	vecsub(r2, r1, dr12);
	vecsub(r3, r2, dr23);
	nrm12[0] = -dr12[1];	nrm12[1] = dr12[0];	// n = ez x dr
	nrm23[0] = -dr23[1];	nrm23[1] = dr23[0];
	normalize(nrm12, nrm12);
	normalize(nrm23, nrm23);
	vecadd(nrm12, nrm23, nrm);	// nrm = (nrm12 + nrm23)/2
	vecdiv(nrm, 2.0, nrm);
	normalize(nrm, nrm);
}


void getnormal2(double r1[2], double r2[2], double r3[2], double nrm[2])	// less stable?
{	// find the outword normal of curve r1-r2-r3 at r2. r1-r2-r3 is CW!
	double dr[2];
	
	vecsub(r3, r1, dr);
	nrm[0] = -dr[1];	// n = ez x dr
	nrm[1] = dr[0];
	normalize(nrm, nrm);
}


double trianglearea(double r1[2], double r2[2], double r3[2])
{	// calc. area of triangle r1-r2-r3
	double r12[2], r13[2], z;
	
	vecsub(r2, r1, r12);
	vecsub(r3, r1, r13);
	z=0.5*fabs(crossprod(r12, r13));
	return z;
}


double triangleareasigndb(double r1[2], double r2[2], double r3[2])
{	// calc. area of triangle r1-r2-r3
	double r12[2], r13[2], z;
	
	vecsub(r2, r1, r12);
	vecsub(r3, r1, r13);
	z=crossprod(r12, r13);
	return z;
}


void getperpfoot(double r1[2], double r2[2], double r3[2], double r4[2])
{	// find r4 which is the perp foot from r3 to line r1-r2
	double k, k2, denom;
	k=(r2[1]-r1[1])/avoidzero(r2[0]-r1[0]);
	k2=k*k;
	denom=1+k2;
	r4[0]=-(-k2*r1[0]-r3[0]+k*(r1[1]-r3[1]))/denom;
	r4[1]=-(k*(r1[0]-r3[0])-r1[1]-k2*r3[1])/denom;
}


void RotMatrix(double sth, double cth, double r1[2], double r2[2])
{
	double rr[2];
	rr[0]=cth*r1[0]-sth*r1[1];
	rr[1]=sth*r1[0]+cth*r1[1];
	veccopy(rr,r2);
}


// segment-segment intersection from "Computational Geometry in C" by O'Rourke
int SegSegLeft(double r1[2], double r2[2], double r3[2])
{
	if(triangleareasigndb(r1, r2, r3) > 0) return 1;	// r3 is on the left of r1->r2
	//if(triangleareasigndb(r1, r2, r3) >= 0) return 1;	// r3 is on the left or on r1->r2
	else return 0;
}


int SegSegCollinear(double r1[2], double r2[2], double r3[2])
{
	if(triangleareasigndb(r1, r2, r3) == 0) return 1;
	else return 0;
}


int Xor(int x, int y)	// exclusive or: =1 iff exactly one argument is true
{
	return !x ^ !y;
}


int SegSegIntersectProp(double r1[2], double r2[2], double r3[2], double r4[2])
{	// return 1 if r1-r2 & r3-r4 intersect, 0 if not
/*
	if(SegSegCollinear(r1,r2,r3) || SegSegCollinear(r1,r2,r4) || 
		SegSegCollinear(r1,r3,r4) || SegSegCollinear(r2,r3,r4)) return 0;
	else return Xor(SegSegLeft(r1,r2,r3), SegSegLeft(r1,r2,r4)) && 
		Xor(SegSegLeft(r1,r3,r4), SegSegLeft(r2,r3,r4));	// full version
*/
	return Xor(SegSegLeft(r1,r2,r3), SegSegLeft(r1,r2,r4)) && 
		Xor(SegSegLeft(r1,r3,r4), SegSegLeft(r2,r3,r4));	// simplified version
}


int PointInCell(double r[2])	// return 1 if point is inside cell, 0 otherwise
{
	int i, cnt;
	double rinf[2];
	MEM_node *m1, *m2;
	
	rinf[0]=XLEN;
	rinf[1]=YLEN;
	cnt=0;
	for(i=0; i<N_MEM; i++) {
		m1 = &MEM[i];
		m2 = &MEM[m1->nbr[1]];
		if(SegSegIntersectProp(r, rinf, m1->r, m2->r)==1) cnt++;	// number of crossings btwn ray and mem
	}
	cnt=cnt%2;	// point is inside if cnt is odd, outside if cnt is even
	return cnt;
}


//----- from NR -----
static float sqrarg;
#define SQR(a) ((sqrarg=(a)) == 0.0 ? 0.0 : sqrarg*sqrarg)
static float maxarg1,maxarg2;
#define FMAX(a,b) (maxarg1=(a),maxarg2=(b),(maxarg1)>(maxarg2)?(maxarg1):(maxarg2))
static float minarg1,minarg2;
#define FMIN(a,b) (minarg1=(a),minarg2=(b),(minarg1)<(minarg2)?(minarg1):(minarg2))

#define NR_END 1
#define FREE_ARG char*

void nrerror(char error_text[])
/* Numerical Recipes standard error handler */
{
	fprintf(stderr,"Numerical Recipes run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	exit(1);
}

unsigned long *lvector(long nl, long nh)
{	// allocate an unsigned long vector with subscript range v[nl..nh]
	unsigned long *v;

	v=(unsigned long *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(long)));
	if (!v) nrerror("allocation failure in lvector()");
	return v-nl+NR_END;
}

double *dvector(long nl, long nh)
{	// allocate a double vector with subscript range v[nl..nh]
	double *v;

	v=(double *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(double)));
	if (!v) nrerror("allocation failure in dvector()");
	return v-nl+NR_END;
}

double **dmatrix(long nrl, long nrh, long ncl, long nch)
{	// allocate a double matrix with subscript range m[nrl..nrh][ncl..nch]
	long i, nrow=nrh-nrl+1,ncol=nch-ncl+1;
	double **m;

	/* allocate pointers to rows */
	m=(double **) malloc((size_t)((nrow+NR_END)*sizeof(double*)));
	if (!m) nrerror("allocation failure 1 in matrix()");
	m += NR_END;
	m -= nrl;

	/* allocate rows and set pointers to them */
	m[nrl]=(double *) malloc((size_t)((nrow*ncol+NR_END)*sizeof(double)));
	if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
	m[nrl] += NR_END;
	m[nrl] -= ncl;

	for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;

	/* return pointer to array of pointers to rows */
	return m;
}

double **convert_dmatrix(double *a, long nrl, long nrh, long ncl, long nch)
/* allocate a float matrix m[nrl..nrh][ncl..nch] that points to the matrix
declared in the standard C manner as a[nrow][ncol], where nrow=nrh-nrl+1
and ncol=nch-ncl+1. The routine should be called with the address
&a[0][0] as the first argument. */
{
	long i,j,nrow=nrh-nrl+1,ncol=nch-ncl+1;
	double **m;

	/* allocate pointers to rows */
	m=(double **) malloc((size_t) ((nrow+NR_END)*sizeof(double*)));
	if (!m) nrerror("allocation failure in convert_matrix()");
	m += NR_END;
	m -= nrl;

	/* set pointers to rows */
	m[nrl]=a-ncl;
	for(i=1,j=nrl+1;i<nrow;i++,j++) m[j]=m[j-1]+ncol;
	/* return pointer to array of pointers to rows */
	return m;
}

void free_lvector(unsigned long *v, long nl, long nh)
{	// free an unsigned long vector allocated with lvector()
	free((FREE_ARG) (v+nl-NR_END));
}

void free_dvector(double *v, long nl, long nh)
{	// free a double vector allocated with dvector()
	free((FREE_ARG) (v+nl-NR_END));
}

void free_dmatrix(double **m, long nrl, long nrh, long ncl, long nch)
{	// free a double matrix allocated by dmatrix()
	free((FREE_ARG) (m[nrl]+ncl-NR_END));
	free((FREE_ARG) (m+nrl-NR_END));
}

void free_convert_dmatrix(double **b, long nrl, long nrh, long ncl, long nch)
{	// free a matrix allocated by convert_matrix()
	free((FREE_ARG) (b+nrl-NR_END));
}

//--- heap sorting ---

void swapdb(double *x,double *y)
{
   double temp;
   temp = *x;
   *x = *y;
   *y = temp;
}

void adjust(double list[], int i, int n)
{
   int j, flag;
   double k;
   
   k = list[i];
   flag = 1;
   j = 2 * i;
   while(j <= n && flag) {
      if(j < n && list[j] < list[j+1]) j++;
      if(k >= list[j]) flag =0;
      else {
         list[j/2] = list[j];
         j = j *2;
      }
   }
   list[j/2] = k;
}

void build_initial_heap(double list[], int n)
{
   int i;
   for(i=(n/2); i>=0; i--) adjust(list, i, n-1);
}

void heapsort(double list[], int n)	// s.t. list is from small to large
{
   int i;
   build_initial_heap(list, n);
   for(i=(n-2); i>=0; i--) {
      swapdb(&list[0], &list[i+1]);
      adjust(list, 0, i);
   }
}

// heapsort2 also sorts the other list according the 1st list

void swapint(int *x,int *y)
{
   int temp;
   temp = *x;
   *x = *y;
   *y = temp;
}

void adjust2(double list[], int list2[], int i, int n)
{
   int j, flag, k2;
   double k;
   
   k = list[i];
   k2 = list2[i];
   flag = 1;
   j = 2 * i;
   while(j <= n && flag) {
      if(j < n && list[j] < list[j+1]) j++;
      if(k >= list[j]) flag =0;
      else {
         list[j/2] = list[j];
		 list2[j/2] = list2[j];
         j = j *2;
      }
   }
   list[j/2] = k;
   list2[j/2] = k2;
}

void build_initial_heap2(double list[], int list2[], int n)
{
   int i;
   for(i=(n/2); i>=0; i--) adjust2(list, list2, i, n-1);
}

void heapsort2(double list[], int list2[], int n)	// s.t. list is from small to large
{
   int i;
   build_initial_heap2(list, list2, n);
   for(i=(n-2); i>=0; i--) {
      swapdb(&list[0], &list[i+1]);
	  swapint(&list2[0], &list2[i+1]);
      adjust2(list, list2, 0, i);
   }
}


//----- RNG: ran1 -----

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

float ran1(long *idum)
{
	int j;
	long k;
	static long iy=0;
	static long iv[NTAB];
	float temp;

	if (*idum <= 0 || !iy) {
		if (-(*idum) < 1) *idum=1;
		else *idum = -(*idum);
		for (j=NTAB+7;j>=0;j--) {
			k=(*idum)/IQ;
			*idum=IA*(*idum-k*IQ)-IR*k;
			if (*idum < 0) *idum += IM;
			if (j < NTAB) iv[j] = *idum;
		}
		iy=iv[0];
	}
	k=(*idum)/IQ;
	*idum=IA*(*idum-k*IQ)-IR*k;
	if (*idum < 0) *idum += IM;
	j=iy/NDIV;
	iy=iv[j];
	iv[j] = *idum;
	if ((temp=AM*iy) > RNMX) return RNMX;
	else return temp;
}
#undef IA
#undef IM
#undef AM
#undef IQ
#undef IR
#undef NTAB
#undef NDIV
#undef EPS
#undef RNMX

      
//----- RNG: ran2 -----

#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
#define NTAB 32
#define NDIV (1+IMM1/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double ran2(long *idum)
{
	int j;
	long k;
	static long idum2=123456789;
	static long iy=0;
	static long iv[NTAB];
	double temp;

	if (*idum <= 0) {
		if (-(*idum) < 1) *idum=1;
		else *idum = -(*idum);
		idum2=(*idum);
		for (j=NTAB+7;j>=0;j--) {
			k=(*idum)/IQ1;
			*idum=IA1*(*idum-k*IQ1)-k*IR1;
			if (*idum < 0) *idum += IM1;
			if (j < NTAB) iv[j] = *idum;
		}
		iy=iv[0];
	}
	k=(*idum)/IQ1;
	*idum=IA1*(*idum-k*IQ1)-k*IR1;
	if (*idum < 0) *idum += IM1;
	k=idum2/IQ2;
	idum2=IA2*(idum2-k*IQ2)-k*IR2;
	if (idum2 < 0) idum2 += IM2;
	j=iy/NDIV;
	iy=iv[j]-idum2;
	iv[j] = *idum;
	if (iy < 1) iy += IMM1;
	if ((temp=AM*iy) > RNMX) return RNMX;
	else return temp;
}
#undef IM1
#undef IM2
#undef AM
#undef IMM1
#undef IA1
#undef IA2
#undef IQ1
#undef IQ2
#undef IR1
#undef IR2
#undef NTAB
#undef NDIV
#undef EPS
#undef RNMX


//----- RNG: ran3 -----

#define MBIG 1000000000
#define MSEED 161803398
#define MZ 0
#define FAC (1.0/MBIG)

double ran3(long *idum)
{
	static int inext,inextp;
	static long ma[56];
	static int iff=0;
	long mj,mk;
	int i,ii,k;

	if (*idum < 0 || iff == 0) {
		iff=1;
		mj=MSEED-(*idum < 0 ? -*idum : *idum);
		mj %= MBIG;
		ma[55]=mj;
		mk=1;
		for (i=1;i<=54;i++) {
			ii=(21*i) % 55;
			ma[ii]=mk;
			mk=mj-mk;
			if (mk < MZ) mk += MBIG;
			mj=ma[ii];
		}
		for (k=1;k<=4;k++)
			for (i=1;i<=55;i++) {
				ma[i] -= ma[1+(i+30) % 55];
				if (ma[i] < MZ) ma[i] += MBIG;
			}
		inext=0;
		inextp=31;
		*idum=1;
	}
	if (++inext == 56) inext=1;
	if (++inextp == 56) inextp=1;
	mj=ma[inext]-ma[inextp];
	if (mj < MZ) mj += MBIG;
	ma[inext]=mj;
	return mj*FAC;
}
#undef MBIG
#undef MSEED
#undef MZ
#undef FAC



//---------------- Exponential distribution --------------------

float expdev(long *idum)
{
	float ran1(long *idum);
	float dum;

	do
		dum=ran1(idum);
	while (dum == 0.0);
	return -log(dum);
}


//---------------- Root Finding --------------------

#define NRANSI
#define ITMAX 100
#define EPS 3.0e-8

double zbrent(double (*func)(double), double x1, double x2, double tol)
{
	int iter;
	double a=x1,b=x2,c=x2,d,e,min1,min2;
	double f_a=(*func)(a),f_b=(*func)(b),f_c,p,q,r,s,tol1,xm;

	if ((f_a > 0.0 && f_b > 0.0) || (f_a < 0.0 && f_b < 0.0))
	//	nrerror("Root must be bracketed in zbrent");
		return 0.0;
	f_c=f_b;
	for (iter=1;iter<=ITMAX;iter++) {
		if ((f_b > 0.0 && f_c > 0.0) || (f_b < 0.0 && f_c < 0.0)) {
			c=a;
			f_c=f_a;
			e=d=b-a;
		}
		if (fabs(f_c) < fabs(f_b)) {
			a=b;
			b=c;
			c=a;
			f_a=f_b;
			f_b=f_c;
			f_c=f_a;
		}
		tol1=2.0*EPS*fabs(b)+0.5*tol;
		xm=0.5*(c-b);
		if (fabs(xm) <= tol1 || f_b == 0.0) return b;
		if (fabs(e) >= tol1 && fabs(f_a) > fabs(f_b)) {
			s=f_b/f_a;
			if (a == c) {
				p=2.0*xm*s;
				q=1.0-s;
			} else {
				q=f_a/f_c;
				r=f_b/f_c;
				p=s*(2.0*xm*q*(q-r)-(b-a)*(r-1.0));
				q=(q-1.0)*(r-1.0)*(s-1.0);
			}
			if (p > 0.0) q = -q;
			p=fabs(p);
			min1=3.0*xm*q-fabs(tol1*q);
			min2=fabs(e*q);
			if (2.0*p < (min1 < min2 ? min1 : min2)) {
				e=d;
				d=p/q;
			} else {
				d=xm;
				e=d;
			}
		} else {
			d=xm;
			e=d;
		}
		a=b;
		f_a=f_b;
		if (fabs(d) > tol1)
			b += d;
		else
			b += SIGN(tol1,xm);
		f_b=(*func)(b);
	}
	//nrerror("Maximum number of iterations exceeded in zbrent");
	return b;
}


double zbrent2(double (*func)(double, double), double val, double x1, double x2, double tol)
{
	int iter;
	double a=x1,b=x2,c=x2,d,e,min1,min2;
	double f_a=(*func)(a,val),f_b=(*func)(b,val),f_c,p,q,r,s,tol1,xm;

	if ((f_a > 0.0 && f_b > 0.0) || (f_a < 0.0 && f_b < 0.0))
	//	nrerror("Root must be bracketed in zbrent");
		return 0.0;
	f_c=f_b;
	for (iter=1;iter<=ITMAX;iter++) {
		if ((f_b > 0.0 && f_c > 0.0) || (f_b < 0.0 && f_c < 0.0)) {
			c=a;
			f_c=f_a;
			e=d=b-a;
		}
		if (fabs(f_c) < fabs(f_b)) {
			a=b;
			b=c;
			c=a;
			f_a=f_b;
			f_b=f_c;
			f_c=f_a;
		}
		tol1=2.0*EPS*fabs(b)+0.5*tol;
		xm=0.5*(c-b);
		if (fabs(xm) <= tol1 || f_b == 0.0) return b;
		if (fabs(e) >= tol1 && fabs(f_a) > fabs(f_b)) {
			s=f_b/f_a;
			if (a == c) {
				p=2.0*xm*s;
				q=1.0-s;
			} else {
				q=f_a/f_c;
				r=f_b/f_c;
				p=s*(2.0*xm*q*(q-r)-(b-a)*(r-1.0));
				q=(q-1.0)*(r-1.0)*(s-1.0);
			}
			if (p > 0.0) q = -q;
			p=fabs(p);
			min1=3.0*xm*q-fabs(tol1*q);
			min2=fabs(e*q);
			if (2.0*p < (min1 < min2 ? min1 : min2)) {
				e=d;
				d=p/q;
			} else {
				d=xm;
				e=d;
			}
		} else {
			d=xm;
			e=d;
		}
		a=b;
		f_a=f_b;
		if (fabs(d) > tol1)
			b += d;
		else
			b += SIGN(tol1,xm);
		f_b=(*func)(b,val);
	}
	//nrerror("Maximum number of iterations exceeded in zbrent");
	return b;
}
#undef ITMAX
#undef EPS
#undef NRANSI

